/***************************************************************************
                          SDLsmtp.c  -  Implements a simple SMTP interface
    layer on top of a TCP socket.  Sends SMTP commands and recieves SMTP
    replies.  Also implements a one-shot email sending function.
                          
                             -------------------
    begin                : Sat May 17 2003
    copyright            : (C) 2003 by Tyler Montbriand
    email                : tsm@accesscomm.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 ***************************************************************************/

#include "SDLsmtp.h"
#include "ipaddr.h"

int SMTPSend(TCPsocket sock, FILE *fp, const char *from,
             const char *to)
{
  char buf[300];

  if(GetMessage(sock)!=SMTP_SERVICEREADY)
  {
    fprintf(stderr,"Server is not ready\n");
    return(-1);
  }

  if(GetIP(NULL,buf,NULL)<0)
  {
    fprintf(stderr,"Couldn't get IP!\n");
    Transact(sock,"QUIT\r\n");
    return(-1);
  }

  if(Transact(sock,"HELO %s\r\n",buf)!=SMTP_OK)
  {
    fprintf(stderr,"Bad reply to HELO\n");
    Transact(sock,"QUIT\r\n");
    return(-1);
  }

  if(Transact(sock,"MAIL FROM:<%s>\r\n",from)!=SMTP_OK)
  {
    fprintf(stderr,"Response to MAIL FROM not SMTP_OK\n");
    Transact(sock,"QUIT\r\n");
    return(-1);
  }

  if(Transact(sock,"RCPT TO:<%s>\r\n",to)!=SMTP_OK)
  {
    fprintf(stderr,"Response to RCPT TO not SMTP_OK\n");
    Transact(sock,"QUIT\r\n");
    return(-1);
  }

  if(Transact(sock,"DATA\r\n")!=SMTP_STARTMAILINPUT)
  {
    fprintf(stderr,"Response to DATA not SMTP_STARTMAIL_INPUT\n");
    Transact(sock,"QUIT\r\n");
    return(-1);
  }

  while(!feof(fp))
  {
    if(fgets(buf,300,fp))
    {
      buf[strlen(buf)-1]='\0';

      if(strcmp(buf,".")==0) strcpy(buf,". ");

      TCPprintf(sock,"%s\r\n",buf);
    }
  }

  if(Transact(sock,".\r\n")!=SMTP_OK)
  {
    fprintf(stderr,"Response to end-of-input not SMTP_OK\n");
    Transact(sock,"QUIT\n");
    return(-1);
  }

  Transact(sock,"QUIT\r\n");

  return(0); 
}

const char *strstart(const char *str, const char *token)
{
  if((str==NULL)||(token==NULL)) return(NULL);
  
  while((*str!='\0')&&(*token!='\0')&&(toupper(*token)==toupper(*str)))
  {
    str++;
    token++;
  }

  if(*token=='\0') return(str);
  else             return(NULL);
}

int LookupMessage(const char *msg)
{
  int n=-1;
  char buf[10];

  for(n=0; n<SMTP_LASTMSG; n++)
  {
    sprintf(buf,"%3d",SMTPNums[n]);

    if(strstart(msg,buf)!=NULL)	return(n);
  }

  return(-1);
}

int GetMessage(TCPsocket sock)
{
  static char buf[1024];
  int pos=0,running=1;

  memset(buf,0,1024);

  while(running)
  {
    char c;
    if(SDLNet_TCP_Recv(sock,&c,1)<0)
      return(-1);

    if(pos<1023)
      buf[pos++]=c;

    if(c=='\n')
      running=0;
  }

  if(sdlsock_debug) fprintf(stderr,"<< %s",buf);

  return(LookupMessage(buf));
}

int Transact(TCPsocket sock, const char *cmdstr, ...)
{
  int retval=-1;
  va_list ap;

  va_start(ap,cmdstr);
  retval=vTCPprintf(sock,cmdstr,ap);
  va_end(ap);

  if(retval<=0)	return(-1);

  else return(GetMessage(sock));
}

int SMTPNums[]={ 211,214,220,221,250,251,354,421,450,451,
                 452,500,501,502,503,504,550,551,552,553,554 };
